@@ -14,7 +14,7 @@
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * ERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
@@ -22,9 +22,16 @@
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
+#include "audiocd.h"
 #include <config-audiocd.h>
+#include <kio_version.h>
+
+#if KIO_VERSION >= QT_VERSION_CHECK(5, 48, 0)
+    #define INSERT(A, B) fastInsert(A, B);
+#else
+    #define INSERT(A, B) insert(A, B);
+#endif
 
-#include <kdemacros.h>
 extern "C"
 {
 	//cdda_interface.h in cdparanoia 10.2 has a member called 'private' which the C++ compiler doesn't like
@@ -33,10 +40,9 @@ extern "C"
 	#include <cdda_paranoia.h>
 	void paranoiaCallback(long, int);
 
-	KDE_EXPORT int kdemain(int argc, char ** argv);
+	int Q_DECL_EXPORT kdemain(int argc, char ** argv);
 }
 
-#include "audiocd.h"
 #include "plugins/audiocdencoder.h"
 
 #include <sys/stat.h>
@@ -45,16 +51,19 @@ extern "C"
 #include <sys/time.h>
 #include <sys/resource.h>
 
-#include <kmacroexpander.h>
+#include <QApplication>
 #include <QFile>
-#include <qfileinfo.h>
-#include <kcmdlineargs.h>
-#include <kdebug.h>
-#include <kapplication.h>
-#include <klocale.h>
-#include <QRegExp>
+#include <QFileInfo>
+#include <QGlobalStatic>
 #include <QHash>
+#include <QRegExp>
+#include <QUrlQuery>
+
+#include "audiocd_kio_debug.h"
+
 #include <KLocalizedString>
+#include <KMacroExpander>
+
 // CDDB
 #include <KCddb/Client>
 #include <KCompactDisc/KCompactDisc>
@@ -65,32 +74,29 @@ using namespace KIO;
 
 using namespace AudioCD;
 
-int kdemain(int argc, char ** argv)
+extern "C" Q_DECL_EXPORT int kdemain(int argc, char **argv)
 {
-	KLocalizedString::setApplicationDomain("kio_audiocd");
-	// KApplication uses libkcddb which needs a valid kapp pointer
-	// GUIenabled must be true as libkcddb sometimes wants to communicate
-	// with the user
-	putenv(strdup("SESSION_MANAGER="));
-	//KApplication::disableAutoDcopRegistration();
-	KCmdLineArgs::init(argc, argv, "kio_audiocd", 0, KLocalizedString(), 0, KLocalizedString());
+    KLocalizedString::setApplicationDomain("kio_audiocd");
+    // QApplication uses libkcddb which needs a valid kapp pointer
+    // GUIenabled must be true as libkcddb sometimes wants to communicate
+    // with the user
+    qunsetenv("SESSION_MANAGER");
+    QApplication app(argc, argv);
+    app.setApplicationName(QStringLiteral("kio_audiocd"));
 
-	KCmdLineOptions options;
-	options.add("+protocol", ki18n("Protocol name"));
-	options.add("+pool", ki18n("Socket name"));
-	options.add("+app", ki18n("Socket name"));
-	KCmdLineArgs::addCmdLineOptions(options);
-	KApplication app(true);
+    if (argc != 4) {
+        fprintf(stderr, "Usage: kio_audiocd protocol pool app\n");
+        exit(-1);
+    }
 
-	kDebug(7117) << "Starting " << getpid();
+    qCDebug(AUDIOCD_KIO_LOG) << "Starting " << getpid();
 
-	KCmdLineArgs* args = KCmdLineArgs::parsedArgs();
-	AudioCDProtocol slave(args->arg(0).toLocal8Bit(), args->arg(1).toLocal8Bit(), args->arg(2).toLocal8Bit());
-	args->clear();
-	slave.dispatchLoop();
+    AudioCDProtocol slave(argv[1], argv[2], argv[3]);
+    slave.dispatchLoop();
 
-	kDebug(7117) << "Done";
-	return 0;
+    qCDebug(AUDIOCD_KIO_LOG) << "Done";
+
+    return 0;
 }
 
 enum Which_dir {
@@ -234,12 +240,12 @@ static void setDeviceToCd(KCompactDisc *cd, struct cdrom_drive *drive)
 		// For ATAPI devices, we have no real choice. Use the
 		// user selected value, even if there is none.
 		//
-		kWarning(7117) << "Found an ATAPI device, assuming it is the one specified by the user.";
+		qCWarning(AUDIOCD_KIO_LOG) << "Found an ATAPI device, assuming it is the one specified by the user.";
 		cd->setDevice( drive->cdda_device_name );
 	}
 	else
 	{
-		kDebug(7117) << "Found a SCSI or ATAPICAM device.";
+		qCDebug(AUDIOCD_KIO_LOG) << "Found a SCSI or ATAPICAM device.";
 		if ( strlen(drive->dev->device_path) > 0 )
 		{
 			cd->setDevice( drive->dev->device_path );
@@ -253,7 +259,7 @@ static void setDeviceToCd(KCompactDisc *cd, struct cdrom_drive *drive)
 			QString devname = QString::fromLatin1( "/dev/%1%2" )
 				.arg( drive->dev->given_dev_name )
 				.arg( drive->dev->given_unit_number ) ;
-			kDebug(7117) << "  Using derived name " << devname;
+			qCDebug(AUDIOCD_KIO_LOG) << "  Using derived name " << devname;
 			cd->setDevice( devname );
 		}
 	}
@@ -406,12 +412,12 @@ struct cdrom_drive * AudioCDProtocol::initRequest(const QUrl & url)
 					break;
 			}
 			// Find where the numbers end
-			for (end = start; end < name.length(); end++)
+			for (end = start; end < name.length(); ++end)
 				if (!name[end].isDigit())
 					break;
 			if (start < name.length()){
 				bool ok;
-				// The external representation counts from 1 so subtrac 1.
+				// The external representation counts from 1 so subtract 1.
 				d->req_track = name.mid(start-1, end - start+2).toInt(&ok) - 1;
 				if (!ok)
 					d->req_track = -1;
@@ -421,7 +427,7 @@ struct cdrom_drive * AudioCDProtocol::initRequest(const QUrl & url)
 	if (d->req_track >= (int)d->tracks)
 		d->req_track = -1;
 
-	kDebug(7117) << "path=" << path << " file=" << d->fname
+	qCDebug(AUDIOCD_KIO_LOG) << "path=" << path << " file=" << d->fname
 		<< " req_track=" << d->req_track << " which_dir=" << d->which_dir << " full CD?=" << d->req_allTracks << endl;
 	return drive;
 }
@@ -544,10 +550,10 @@ void AudioCDProtocol::stat(const QUrl & url)
 		const mode_t _umask = ::umask(0);
 		::umask(_umask);
 		UDSEntry entry;
-		entry.insert(KIO::UDSEntry::UDS_NAME, url.fileName().replace(QLatin1Char( '/' ), QLatin1String("%2F")));
-		entry.insert(KIO::UDSEntry::UDS_FILE_TYPE, S_IFDIR);
-		entry.insert(KIO::UDSEntry::UDS_ACCESS, (0666 & (~_umask)));
-		entry.insert(KIO::UDSEntry::UDS_SIZE, 2+encoders.count());
+		entry.INSERT(KIO::UDSEntry::UDS_NAME, url.fileName().replace(QLatin1Char( '/' ), QLatin1String("%2F")));
+		entry.INSERT(KIO::UDSEntry::UDS_FILE_TYPE, S_IFDIR);
+		entry.INSERT(KIO::UDSEntry::UDS_ACCESS, (0666 & (~_umask)));
+		entry.INSERT(KIO::UDSEntry::UDS_SIZE, 2+encoders.count());
 		statEntry(entry);
 		finished();
 		return;
@@ -577,26 +583,26 @@ void AudioCDProtocol::stat(const QUrl & url)
 
 	UDSEntry entry;
 
-	entry.insert( KIO::UDSEntry::UDS_NAME, url.fileName().replace(QLatin1Char( '/' ), QLatin1String("%2F") ));
+	entry.INSERT( KIO::UDSEntry::UDS_NAME, url.fileName().replace(QLatin1Char( '/' ), QLatin1String("%2F") ));
 
-	entry.insert( KIO::UDSEntry::UDS_FILE_TYPE,isFile ? S_IFREG : S_IFDIR);
+	entry.INSERT( KIO::UDSEntry::UDS_FILE_TYPE,isFile ? S_IFREG : S_IFDIR);
 
 
 	const mode_t _umask = ::umask(0);
 	::umask(_umask);
 
-	entry.insert(KIO::UDSEntry::UDS_ACCESS, (0666 & (~_umask)));
+	entry.INSERT(KIO::UDSEntry::UDS_ACCESS, (0666 & (~_umask)));
 
 	if (!isFile)
 	{
-		entry.insert( KIO::UDSEntry::UDS_SIZE, cdda_tracks(drive));
+		entry.INSERT( KIO::UDSEntry::UDS_SIZE, cdda_tracks(drive));
 	}
 	else
 	{
 			AudioCDEncoder *encoder = determineEncoder(d->fname);
-			long firstSector, lastSector;
+			long firstSector = 0, lastSector = 0;
 			getSectorsForRequest(drive, firstSector, lastSector);
-			entry.insert( KIO::UDSEntry::UDS_SIZE,fileSize(firstSector, lastSector, encoder));
+			entry.INSERT( KIO::UDSEntry::UDS_SIZE,fileSize(firstSector, lastSector, encoder));
 	}
 
 
@@ -610,22 +616,22 @@ void AudioCDProtocol::stat(const QUrl & url)
 static void app_dir(UDSEntry& e, const QString & n, size_t s)
 {
 	e.clear();
-	e.insert( KIO::UDSEntry::UDS_NAME, QFile::decodeName(n.toLocal8Bit()));
-	e.insert( KIO::UDSEntry::UDS_FILE_TYPE, S_IFDIR);
-	e.insert( KIO::UDSEntry::UDS_ACCESS, 0400);
-	e.insert( KIO::UDSEntry::UDS_SIZE, s);
-	e.insert( KIO::UDSEntry::UDS_MIME_TYPE, QLatin1String("inode/directory"));
+	e.INSERT( KIO::UDSEntry::UDS_NAME, QFile::decodeName(n.toLocal8Bit()));
+	e.INSERT( KIO::UDSEntry::UDS_FILE_TYPE, S_IFDIR);
+	e.INSERT( KIO::UDSEntry::UDS_ACCESS, 0400);
+	e.INSERT( KIO::UDSEntry::UDS_SIZE, s);
+	e.INSERT( KIO::UDSEntry::UDS_MIME_TYPE, QLatin1String("inode/directory"));
 }
 
 static void app_file(UDSEntry& e, const QString & n, size_t s, const QString &mimetype = QString())
 {
 	e.clear();
-	e.insert( KIO::UDSEntry::UDS_NAME, QFile::decodeName(n.toLocal8Bit()));
-	e.insert( KIO::UDSEntry::UDS_FILE_TYPE, S_IFREG);
-	e.insert( KIO::UDSEntry::UDS_ACCESS, 0400);
-	e.insert( KIO::UDSEntry::UDS_SIZE, s);
+	e.INSERT( KIO::UDSEntry::UDS_NAME, QFile::decodeName(n.toLocal8Bit()));
+	e.INSERT( KIO::UDSEntry::UDS_FILE_TYPE, S_IFREG);
+	e.INSERT( KIO::UDSEntry::UDS_ACCESS, 0400);
+	e.INSERT( KIO::UDSEntry::UDS_SIZE, s);
 	if (!mimetype.isEmpty())
-		e.insert( KIO::UDSEntry::UDS_MIME_TYPE, mimetype);
+		e.INSERT( KIO::UDSEntry::UDS_MIME_TYPE, mimetype);
 }
 
 void AudioCDProtocol::listDir(const QUrl & url)
@@ -639,14 +645,15 @@ void AudioCDProtocol::listDir(const QUrl & url)
 		foreach (const QString &deviceName, deviceNames) {
 			const QString &device = KCompactDisc::urlToDevice(KCompactDisc::cdromDeviceUrl(deviceName));
 			QUrl targetUrl = url;
-			targetUrl.addEncodedQueryItem("device", device.toUtf8());
+			QUrlQuery targetQuery;
+			targetQuery.addQueryItem("device", device.toUtf8());
+			targetUrl.setQuery(targetQuery);
 			app_dir(entry, device, 2+encoders.count());
-			entry.insert(KIO::UDSEntry::UDS_TARGET_URL, targetUrl.url());
-			entry.insert(KIO::UDSEntry::UDS_DISPLAY_NAME, deviceName);
-			listEntry(entry, false);
+			entry.INSERT(KIO::UDSEntry::UDS_TARGET_URL, targetUrl.url());
+			entry.INSERT(KIO::UDSEntry::UDS_DISPLAY_NAME, deviceName);
+			listEntry(entry);
 		}
 		totalSize(entry.count());
-		listEntry(entry, true);
 		finished();
 		return;
 	}
@@ -685,24 +692,24 @@ void AudioCDProtocol::listDir(const QUrl & url)
 			else
                             app_file(entry, QString::fromLatin1("%1_%2.txt").arg(i18n(CDDB_INFORMATION)).arg(count), ((*it).toString().length())+1);
 			count++;
-			listEntry(entry, false);
+			listEntry(entry);
 		}
 		// Error
 		if( count == 1 ) {
                     app_file(entry, QString::fromLatin1("%1: %2.txt").arg(i18n(CDDB_INFORMATION)).arg(KCDDB::resultToString(d->cddbResult)), 0);
 			count++;
-			listEntry(entry, false);
+			listEntry(entry);
 		}
 	}
 
 	if (d->which_dir == Root){
 		// List virtual directories.
 		app_dir(entry, d->s_fullCD, encoders.count());
-		listEntry(entry, false);
+		listEntry(entry);
 
 		// Either >0 cddb results or cddb error file
 		app_dir(entry, d->s_info, d->cddbList.count());
-		listEntry(entry, false);
+		listEntry(entry);
 
 		// List the encoders
 		AudioCDEncoder *encoder;
@@ -712,7 +719,7 @@ void AudioCDProtocol::listDir(const QUrl & url)
 			if( encoder == encoderTypeWAV )
 				continue;
 			app_dir(entry, encoder->type(), d->tracks);
-			listEntry(entry, false);
+			listEntry(entry);
 		}
 	}
 
@@ -757,12 +764,11 @@ void AudioCDProtocol::listDir(const QUrl & url)
 		else
 		{
 			app_dir(entry, d->child_dir, 1);
-			listEntry(entry, false);
+			listEntry(entry);
 		}
 	}
 
 	totalSize(entry.count());
-	listEntry(entry, true);
 	cdda_close(drive);
 	finished();
 }
@@ -787,7 +793,7 @@ void AudioCDProtocol::addEntry(const QString& trackTitle, AudioCDEncoder *encode
 	}
 	UDSEntry entry;
 	app_file(entry, trackTitle + QLatin1String(".")+QLatin1String( encoder->fileType() ), theFileSize, QLatin1String( encoder->mimeType() ));
-	listEntry(entry, false);
+	listEntry(entry);
 }
 
 long AudioCDProtocol::fileSize(long firstSector, long lastSector, AudioCDEncoder *encoder)
@@ -813,7 +819,7 @@ struct cdrom_drive *AudioCDProtocol::getDrive()
 	drive = cdda_identify(device, CDDA_MESSAGE_FORGETIT, 0);
 
 	if (0 == drive) {
-		kDebug(7117) << "Can't find an audio CD on: \"" << d->device << "\"";
+		qCDebug(AUDIOCD_KIO_LOG) << "Can't find an audio CD on: \"" << d->device << "\"";
 
 		const QFileInfo fi(d->device);
 		if(!fi.isReadable())
@@ -829,7 +835,7 @@ i18n("Unknown error.  If you have a cd in the drive try running cdparanoia -vsQ
 
 	if (0 != cdda_open(drive))
 	{
-		kDebug(7117) << "cdda_open failed";
+		qCDebug(AUDIOCD_KIO_LOG) << "cdda_open failed";
 		error(KIO::ERR_CANNOT_OPEN_FOR_READING, d->device);
 		cdda_close(drive);
 		return 0;
@@ -852,7 +858,7 @@ void AudioCDProtocol::paranoiaRead(
 
 	cdrom_paranoia * paranoia = paranoia_init(drive);
 	if (0 == paranoia) {
-		kDebug(7117) << "paranoia_init failed";
+		qCDebug(AUDIOCD_KIO_LOG) << "paranoia_init failed";
 		return;
 	}
 
@@ -895,13 +901,13 @@ void AudioCDProtocol::paranoiaRead(
 	while (currentSector <= lastSector)
 	{
 		// TODO make the 5 configurable? The default in the lib is 20 fyi
-		int16_t * buf = paranoia_read_limited(paranoia, paranoiaCallback, 5);
+		qint16 * buf = paranoia_read_limited(paranoia, paranoiaCallback, 5);
 		if( warned == 0 && paranoia_read_limited_error >= 5 && d->reportErrors ){
 			warning(i18n("AudioCD: Disk damage detected on this track, risk of data corruption."));
 			warned = 1;
 		}
 		if (0 == buf) {
-			kDebug(7117) << "Unrecoverable error in paranoia_read";
+			qCDebug(AUDIOCD_KIO_LOG) << "Unrecoverable error in paranoia_read";
 			ok = false;
 			error( ERR_SLAVE_DEFINED, i18n( "Error reading audio data for %1 from the CD", fileName ) );
 			break;
@@ -911,7 +917,7 @@ void AudioCDProtocol::paranoiaRead(
 
 		int encoderProcessed = encoder->read(buf, CD_FRAMESAMPLES);
 		if(encoderProcessed == -1){
-			kDebug(7117) << "Encoder processing error, stopping.";
+			qCDebug(AUDIOCD_KIO_LOG) << "Encoder processing error, stopping.";
 			ok = false;
 			QString errMsg = i18n( "Could not read %1: encoding failed", fileName );
 			QString details = encoder->lastErrorMessage();
@@ -1005,7 +1011,7 @@ void AudioCDProtocol::parseURLArgs(const QUrl & url)
 {
 	d->clearURLargs();
 
-	QString query(QUrl::fromPercentEncoding(url.query().toAscii()));
+	QString query(QUrl::fromPercentEncoding(url.query().toLatin1()));
 
 	if (query.isEmpty())
 		return;
@@ -1038,7 +1044,7 @@ void AudioCDProtocol::parseURLArgs(const QUrl & url)
 		else if (attribute == QLatin1String("niceLevel")){
 			int niceLevel = value.toInt();
 			if(setpriority(PRIO_PROCESS, getpid(), niceLevel) != 0)
-				kDebug(7117) << "Setting nice level to (" << niceLevel << ") failed.";
+				qCDebug(AUDIOCD_KIO_LOG) << "Setting nice level to (" << niceLevel << ") failed.";
 		}
 	}
 }
@@ -1069,7 +1075,7 @@ void AudioCDProtocol::loadSettings()
 	if(groupCDDA.hasKey("niceLevel")) {
 		int niceLevel = groupCDDA.readEntry("niceLevel", 0);
 		if(setpriority(PRIO_PROCESS, getpid(), niceLevel) != 0)
-			kDebug(7117) << "Setting nice level to (" << niceLevel << ") failed.";
+			qCDebug(AUDIOCD_KIO_LOG) << "Setting nice level to (" << niceLevel << ") failed.";
 	}
 
 	// The default track filename template
@@ -1162,60 +1168,60 @@ void AudioCDProtocol::generateTemplateTitles()
  * Based upon the cdparanoia ripping application
  * Only output BAD stuff
  * The higher the paranoia_read_limited_error the worse the problem is
- * FYI: PARANOIA_CB_READ & PARANOIA_CB_VERIFY happen continusly when ripping
+ * FYI: PARANOIA_CB_READ & PARANOIA_CB_VERIFY happen continuously when ripping
  */
 void paranoiaCallback(long, int function)
 {
 	switch(function){
 		case PARANOIA_CB_VERIFY:
-			//kDebug(7117) << "PARANOIA_CB_VERIFY";
+			//qCDebug(AUDIOCD_KIO_LOG) << "PARANOIA_CB_VERIFY";
 			break;
 
 		case PARANOIA_CB_READ:
-			//kDebug(7117) << "PARANOIA_CB_READ";
+			//qCDebug(AUDIOCD_KIO_LOG) << "PARANOIA_CB_READ";
 			break;
 
 		case PARANOIA_CB_FIXUP_EDGE:
-			//kDebug(7117) << "PARANOIA_CB_FIXUP_EDGE";
+			//qCDebug(AUDIOCD_KIO_LOG) << "PARANOIA_CB_FIXUP_EDGE";
 			paranoia_read_limited_error = 2;
 			break;
 
 		case PARANOIA_CB_FIXUP_ATOM:
-			//kDebug(7117) << "PARANOIA_CB_FIXUP_ATOM";
+			//qCDebug(AUDIOCD_KIO_LOG) << "PARANOIA_CB_FIXUP_ATOM";
 			paranoia_read_limited_error = 6;
 			break;
 
 		case PARANOIA_CB_READERR:
-			kDebug(7117) << "PARANOIA_CB_READERR";
+			qCDebug(AUDIOCD_KIO_LOG) << "PARANOIA_CB_READERR";
 			paranoia_read_limited_error = 6;
 			break;
 
 		case PARANOIA_CB_SKIP:
-			kDebug(7117) << "PARANOIA_CB_SKIP";
+			qCDebug(AUDIOCD_KIO_LOG) << "PARANOIA_CB_SKIP";
 			paranoia_read_limited_error = 8;
 			break;
 
 		case PARANOIA_CB_OVERLAP:
-			//kDebug(7117) << "PARANOIA_CB_OVERLAP";
+			//qCDebug(AUDIOCD_KIO_LOG) << "PARANOIA_CB_OVERLAP";
 			break;
 
 		case PARANOIA_CB_SCRATCH:
-			kDebug(7117) << "PARANOIA_CB_SCRATCH";
+			qCDebug(AUDIOCD_KIO_LOG) << "PARANOIA_CB_SCRATCH";
 			paranoia_read_limited_error = 7;
 			break;
 
 		case PARANOIA_CB_DRIFT:
-			//kDebug(7117) << "PARANOIA_CB_DRIFT";
+			//qCDebug(AUDIOCD_KIO_LOG) << "PARANOIA_CB_DRIFT";
 			paranoia_read_limited_error = 4;
 			break;
 
 		case PARANOIA_CB_FIXUP_DROPPED:
-			kDebug(7117) << "PARANOIA_CB_FIXUP_DROPPED";
+			qCDebug(AUDIOCD_KIO_LOG) << "PARANOIA_CB_FIXUP_DROPPED";
 			paranoia_read_limited_error = 5;
 			break;
 
 		case PARANOIA_CB_FIXUP_DUPED:
-			kDebug(7117) << "PARANOIA_CB_FIXUP_DUPED";
+			qCDebug(AUDIOCD_KIO_LOG) << "PARANOIA_CB_FIXUP_DUPED";
 			paranoia_read_limited_error = 5;
 			break;
 	}
